from easydict import EasyDict as edict

class Service:

    def __init__(self, module):
        self.module = module
        self.map = map(module)

    def execute (self, url, args = []):
        try:
            method = self.map[url]
        except:
            raise { "statusCode": 404, "code": 'METHOD_NOT_FOUND', "method": url }
        
        return method(*args)

def map (module, base_route = ''):
    routes = edict({})

    for i in module:
        sub_module = module[i]
        route = base_route + '/' + i

        if callable(sub_module):
            routes[route] = sub_module
        elif isinstance(sub_module, edict):
            routes = { **routes, **map(sub_module, route)}
        else:
            routes[route] = (lambda _sub_module: (lambda: _sub_module))(sub_module)
    
    keys = list(routes.keys())
    
    if not base_route:
        routes['/'] = lambda: keys
    
    return routes


import service
import json

from http.server import BaseHTTPRequestHandler, HTTPServer
from easydict import EasyDict as edict

class Router:

    def __init__ (self, module, config = {}):
        config = edict(config)

        self.service = service.Service(module)
        try:
            self.host = config.host
        except:
            self.host = "0.0.0.0"
         
        self.port = config.port
    
    def route (self, url, args):
        return self.service.execute(url, args)
    
    def serve (self):
        self.server = HTTPServer((self.host, self.port), self.Handler())

        print("Server running at: http://" + self.host + ":" + str(self.port))

        self.server.serve_forever()
    
    def Handler (self):

        class BaseHandler(BaseHTTPRequestHandler):
            def do_GET (sock):
                sock.send_response(200)

                sock.send_header('Content-type','application/json')
                sock.end_headers()

                sock.wfile.write(bytes(json.dumps({
                    "status": "err",
                    "result": "METHOD_UNSUPPORTED"
                }), "utf8"))
        
            def do_POST (sock):
                _json = {}

                try:
                    str = ""
                    data = sock.rfile.read(int(sock.headers['content-length']))
                    args = json.loads(data.decode("utf8"))

                    _json = { "status": "ok", "result": self.route(sock.path, args)}
                except Exception as exc:
                    _json = { "status": "err", "result": exc.args }
                
                sock.send_response(200)

                sock.send_header('Content-type','application/json')
                sock.end_headers()

                sock.wfile.write(bytes(json.dumps(_json), "utf8"))
                sock.connection.close()

        return BaseHandler


from distutils.core import setup
setup(
  name = 'unete',         
  packages = ['unete'],   
  version = '1.0',      
  license='MIT',        
  description = 'Unete client for Python, access remote methods with native syntax.',   
  author = 'CamiloTD',                   
  author_email = 'camilotd1999@gmail.com',      
  url = 'https://github.com/CamiloTD/unete-python',   
  download_url = 'https://github.com/CamiloTD/unete-python/archive/1.0.tar.gz',    
  keywords = ['unete', 'microservices', 'remote', 'native', 'api'],   
  install_requires=['edict'],
  classifiers=[
    'Development Status :: 3 - Alpha',      
    'Intended Audience :: Developers',      
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   
    'Programming Language :: Python :: 3',      
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)


from easydict import EasyDict as edict
import http.client
import json

class Connector:
    
    def __init__(self, host):
        self.host = host
        self.map  = edict({})
        self.client = http.client.HTTPConnection(self.host)
    
    def init (self):
        map = self.post('/')
        self.map = self.unmap(map)

    def post (self, method, data = [] ):
        hrequest = self.client.request('POST', method, json.dumps(data))
        hresponse = self.client.getresponse()
        
        result = edict(json.load(hresponse))

        if result.status != 'ok':
            raise Exception(result.result)
        try:
            return result.result
        except:
            return
    
    def unmap (self, array = []):
        map = edict({})

        for url in array:
            keys = url.split('/')
            ref = map

            for i in range(0, len(keys) - 1):
                key = keys[i]
                if not key:
                    continue
                try:
                    ref = map[key]
                except:
                    ref = edict({})
                    map[key] = ref
                
            key = keys[len(keys) - 1]
            
            ref[key] = (lambda _url: lambda *data: self.post(_url, data))(url)
        
        return map

def Connect(host):
    connector = Connector(host)
    connector.init()
    return connector.map